/*
 *
 * AttributeMgr
 *
 * Central point for the attribute skill system in the mod
 *
 *
*/

package jesse.keeblarcraft.AttributeMgr;

import java.util.HashMap;

import jesse.keeblarcraft.Keeblarcraft;
import jesse.keeblarcraft.AttributeMgr.AttributeNodes.AbstractNode;
import jesse.keeblarcraft.AttributeMgr.AttributeNodes.AttributeFlight;
import jesse.keeblarcraft.AttributeMgr.AttributeNodes.AttributeMetalJacket;
import jesse.keeblarcraft.ConfigMgr.ConfigManager;
import jesse.keeblarcraft.Utils.ChatUtil;

public class AttributeMgr {
    ConfigManager config;

    // Global list of attributes
    // Key: node title
    // Val: The node class itself
    public static final HashMap<String, Class<? extends AbstractNode>> attributes = new HashMap<String, Class<? extends AbstractNode>>();

    // This is a list of all logged in player tree's. These are kept in memory until either the mod is torn down or
    // players log out for optimization reasons
    public static final HashMap<String, AttributeTree> activeTrees = new HashMap<String, AttributeTree>();

    /////////////////////////////////////////////////////////////////////////////
    /// @fn         RegisterAttributeClass
    ///
    /// @brief      All generic class objects for attributes should be
    ///             registered via this function
    /////////////////////////////////////////////////////////////////////////////
    @SuppressWarnings("deprecation")
    public static void RegisterAttributeClass(Class<? extends AbstractNode> classObj) {
        AbstractNode verifyNode = null;
        try {
            // We spin up an instance of this node; but it will be dead after this function. We need this for name
            verifyNode = classObj.newInstance();
        } catch (Exception e) {
            Keeblarcraft.LOGGER.error("Attempted to assign AbstractNode class type when registering object but could not call .newInstance()! Constructs must be empty");
            e.printStackTrace();
        }
        ChatUtil.LoggerColored("Registring attribute called", ChatUtil.CONSOLE_COLOR.CYAN, Keeblarcraft.LOGGER);

        try {
            if (attributes.containsKey(verifyNode.GetNodeTitle())) {
                Keeblarcraft.LOGGER.warn("Could not register attribute with duplicate name '" + verifyNode.GetNodeTitle() + "'");
            } else {
                ChatUtil.LoggerColored("REGISTERING ATTRIBUTE " + verifyNode.GetNodeTitle(), ChatUtil.CONSOLE_COLOR.YELLOW, Keeblarcraft.LOGGER);
                attributes.put(verifyNode.GetNodeTitle(), classObj);
            }
        } catch (Exception e) {} // Left empty since previous try-catch will throw error-message
    }

    /////////////////////////////////////////////////////////////////////////////
    /// @fn         ApplyAttribute
    ///
    /// @arg[in]    uuid is the players uuid
    ///
    /// @arg[in]    attributeName is the node title of the attribute class object
    ///
    /// @brief      Used to apply an attribute to a player's attribute tree
    /////////////////////////////////////////////////////////////////////////////
    @SuppressWarnings("deprecation")
    public static String ApplyAttribute(String uuid, String attributeName) {
        String msg = "";
        if (attributes.containsKey(attributeName)) {
            AttributeTree playerTree = activeTrees.get(uuid);
            AbstractNode node = null;
            try {
                node = attributes.get(attributeName).newInstance();
            } catch (Exception e) {
                Keeblarcraft.LOGGER.error("Could not successfully apply attribute. String of attribute name could not be successfully cast to actual java object");
            }

            if (playerTree != null && node != null) {
                ///////////
                // debug testing
                String isNull = (node == null ? "NULL" : "NOT NULL");
                System.out.println("Node is " + isNull);
                if (isNull.equals("NOT NULL")) { System.out.println("Node name: " + node.GetNodeTitle()); }
                // end debug testing
                ///////////

                playerTree.AddNewNode(node, 1, null, null);
                msg = "Applied attribute '" + attributeName + "' successfully";
            } else {
                msg = "Player tree not found!";
            }
        } else {
            msg = "Could not apply attribute, attribute name does not exist!";
        }
        return msg;
    }

    /////////////////////////////////////////////////////////////////////////////
    /// @fn         RegisterAttributes
    ///
    /// @brief      This registers all attribute classes to the global attribute
    ///             class. Individual classes can be added here that call the
    ///             more braod "RegisterAttributeClass"
    ///             @see RegisterAttributeClass
    /////////////////////////////////////////////////////////////////////////////
    public static void RegisterAttributes() {
        // Manually register all attribute node classes here
        /// TODO: Find a better way to do this more dynamically in the future
        /// hint: make it an API for other modders to add to

        RegisterAttributeClass(AttributeFlight.class);
        RegisterAttributeClass(AttributeMetalJacket.class);
    }
}
